package de.tudresden.inf.st.ros2rag.example;

import com.google.protobuf.InvalidProtocolBufferException;
import de.tudresden.inf.st.ros2rag.ast.*;
import panda.Linkstate.PandaLinkState.Position;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

/**
 * Testing Ros2Rag without generating something.
 *
 * @author rschoene - Initial contribution
 */
public class Main {
  public static void main(String[] args) throws InvalidProtocolBufferException, IOException, InterruptedException {
    Model model = new Model();
    model.updateMqttHost("localhost");

    ZoneModel zoneModel = new ZoneModel();
    zoneModel.setSize(makePosition(1, 1, 1));

    Position myPosition = makePosition(0, 0, 0);
    PositionWrapper myPositionWrapper = new PositionWrapper(myPosition);
    PositionWrapper leftPosition = new PositionWrapper(makePosition(-1, 0, 0));
    PositionWrapper rightPosition = new PositionWrapper(makePosition(1, 0, 0));

    Zone safetyZone = new Zone();
    safetyZone.addPositionWrapper(myPositionWrapper);
    safetyZone.addPositionWrapper(leftPosition);
    safetyZone.addPositionWrapper(rightPosition);
    zoneModel.addSafetyZone(safetyZone);

    RobotArm robotArm = new GeneratedRobotArm();

    GeneratedJoint joint1 = new GeneratedJoint();
    joint1.setName("joint1");
    joint1.setCurrentPosition(myPosition);

    EndEffector endEffector = new EndEffector();
    endEffector.setName("gripper");
    endEffector.setCurrentPosition(myPosition);

    robotArm.addJoint(joint1);
    robotArm.setEndEffector(endEffector);
    model.setRobotArm(robotArm);

    model.waitUntilReady(2, TimeUnit.SECONDS);

    joint1.connectCurrentPosition("robot/joint1");
    System.out.println("BEFORE joint1.getCurrentPosition() = " + stringify(joint1.getCurrentPosition()));

    Thread.sleep(10000);

    System.out.println("AFTER joint1.getCurrentPosition() = " + stringify(joint1.getCurrentPosition()));

    // TODO close/shutdown should be exposed
    model.get_MqttRoot().getUpdater().close();
  }

  private static Position makePosition(int x, int y, int z) {
    return Position.newBuilder()
        .setPositionX(x)
        .setPositionY(y)
        .setPositionZ(z)
        .build();
  }

  private static String stringify(Position position) {
    return "(" + position.getPositionX() + ", " + position.getPositionY() + ", " + position.getPositionZ() + ")";
  }
}
